@@ -24,6 +24,13 @@ class SimplePropertiesTests(unittest.TestCase):
         self.assertAlmostEqual(7 / 16, self.gdir.density(True), places=5)
         self.assertAlmostEqual(1 / 7, self.tree.density(), places=5)
 
+    def testMeanDegree(self):
+        self.assertEqual(9.0, self.gfull.mean_degree())
+        self.assertEqual(0.0, self.gempty.mean_degree())
+        self.assertEqual(2.5, self.g.mean_degree())
+        self.assertEqual(7 / 4, self.gdir.mean_degree())
+        self.assertAlmostEqual(13 / 7, self.tree.mean_degree(), places=5)
+
     def testDiameter(self):
         self.assertTrue(self.gfull.diameter() == 1)
         self.assertTrue(self.gempty.diameter(unconn=False) == inf)
@@ -144,10 +151,6 @@ class DegreeTests(unittest.TestCase):
         self.assertTrue(knn == [9.0] * 10)
         self.assertAlmostEqual(knnk[8], 9.0, places=6)
 
-        # knn works for simple graphs only -- self.g is not simple
-        self.assertRaises(InternalError, self.g.knn)
-
-        # Okay, simplify it and then go on
         g = self.g.copy()
         g.simplify()
 
@@ -158,6 +161,20 @@ class DegreeTests(unittest.TestCase):
         self.assertAlmostEqual(knnk[1], 3, places=6)
         self.assertAlmostEqual(knnk[2], 7 / 3.0, places=6)
 
+    def testKnnNonSimple(self):
+        knn, knnk = self.gfull.knn()
+        self.assertTrue(knn == [9.0] * 10)
+        self.assertAlmostEqual(knnk[8], 9.0, places=6)
+
+        # knn works for non-simple graphs as well
+        knn, knnk = self.g.knn()
+        diff = max(abs(a - b) for a, b in zip(knn, [17 / 5.0, 3, 4, 4]))
+        self.assertAlmostEqual(diff, 0.0, places=6)
+        self.assertEqual(len(knnk), 5)
+        self.assertAlmostEqual(knnk[1], 4, places=6)
+        self.assertAlmostEqual(knnk[2], 3, places=6)
+        self.assertAlmostEqual(knnk[4], 3.4, places=6)
+
     def testDegree(self):
         self.assertTrue(self.gfull.degree() == [9] * 10)
         self.assertTrue(self.gempty.degree() == [0] * 10)
@@ -232,6 +249,8 @@ class BiconnectedComponentTests(unittest.TestCase):
     g1 = Graph.Full(10)
     g2 = Graph(5, [(0, 1), (1, 2), (2, 3), (3, 4)])
     g3 = Graph(6, [(0, 1), (1, 2), (2, 3), (3, 0), (2, 4), (2, 5), (4, 5)])
+    g4 = Graph.Full(2)
+    g5 = Graph.Full(1)
 
     def testBiconnectedComponents(self):
         s = self.g1.biconnected_components()
@@ -250,6 +269,14 @@ class BiconnectedComponentTests(unittest.TestCase):
         self.assertTrue(self.g1.articulation_points() == [])
         self.assertTrue(self.g2.cut_vertices() == [1, 2, 3])
         self.assertTrue(self.g3.articulation_points() == [2])
+        self.assertTrue(self.g4.articulation_points() == [])
+
+    def testIsBiconnected(self):
+        self.assertTrue(self.g1.is_biconnected())
+        self.assertFalse(self.g2.is_biconnected())
+        self.assertFalse(self.g3.is_biconnected())
+        self.assertTrue(self.g4.is_biconnected())
+        self.assertFalse(self.g5.is_biconnected())
 
 
 class CentralityTests(unittest.TestCase):
@@ -974,7 +1001,7 @@ class PathTests(unittest.TestCase):
             self.assertEqual(15, path[-1])
             curr = path[0]
             for next in path[1:]:
-                self.assertTrue(g.are_connected(curr, next))
+                self.assertTrue(g.are_adjacent(curr, next))
                 curr = next
 
     def testPathLengthHist(self):
